"use strict";exports.id=490,exports.ids=[490],exports.modules={6001:(e,r,i)=>{Object.defineProperty(r,"__esModule",{value:!0}),r.loadLibrary=function(e){let r=o[e];if("string"!=typeof r){let i;i="jquery"===e?s:(0,t.join)(a,e);try{r=(0,n.readFileSync)(i).toString()}catch(t){console.log(`Unable to load library ${e} at ${i}`),r=""}o[e]=r}return r};const t=i(6928),n=i(9896),o={},d="dist"===(0,t.basename)(__dirname)?(0,t.dirname)(__dirname):(0,t.dirname)((0,t.dirname)(__dirname)),a=(0,t.join)(d,"../../node_modules/typescript/lib"),s=(0,t.join)(d,"lib/jquery.d.ts")}};
//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/c9d77990917f3102ada88be140d28b038d1dd7c7/extensions/html-language-features/server/dist/node/490.htmlServerMain.js.map